#include "stdInclude.h"

class PictureServer
{
	StreamSocket* listener;
	StreamSocket* client;
	Lock* lock;
	int connectionPort;
	bool shutdown;

	static void receiveThread(void* server)
	{
		((PictureServer*)server)->receive();
	}

	void receive()
	{
		while(!shutdown)
		{
			try
			{
				lock->aquire();
				listener = new StreamSocket();
				lock->release();

				listener->open();
				listener->bind(SocketAddress(SocketAddress::ANY, connectionPort));
				printd(INFO,"listening for picture client ...\n");
				listener->listen(1);
				StreamSocket socket = listener->accept();
				
				lock->aquire();
				client = new StreamSocket(socket);
				listener->close();
				delete listener;
				listener = NULL;
				lock->release();
				printd(INFO,"picture client connected\n");
			}
			catch(...)
			{
				lock->aquire();
				delete listener;
				listener = NULL;
				lock->release();
				printd(INFO,"listening for picture client stopped\n");
				return;
			}
			try
			{		
				while(!shutdown)
				{
					unsigned msgSize, msgSizeNet;
					uint8_t* buffer;
					NetMessage message;
					client->recv(&msgSizeNet, 4);
					msgSize = ntohl(msgSizeNet);
					buffer = message.allocateAtEnd(msgSize);
					client->recv(buffer, msgSize);
					printd(INFO,"got new picture\n");
					messageReceived(&message);
				}
			}
			catch(...)
			{}

			lock->aquire();
			delete client;
			client = NULL;
			lock->release();
			printd(INFO,"picture client disconnected\n");
		}
	}
	
	virtual void messageReceived(NetMessage* message) = 0;
public:
	void accept(int port)
	{
		shutdown = false;
		listener = NULL;
		client = NULL;
		connectionPort = port;
		lock = ThreadManager::the()->getLock("PictureServerLock");
		Thread* serverThread = dynamic_cast<Thread *> (ThreadManager::the()->getThread("PictureServerThread"));
		serverThread->runFunction(PictureServer::receiveThread, 0, this);
	}

	void close()
	{
		lock->aquire();
		if(client) client->close();
		if(listener) listener->close();
		shutdown = true;
		lock->release();
	}
};